<?php
/**
 * eXtreme Message Board
 * XMB 1.9.11
 *
 * Sitemap Protocol Add-On
 *   Version 1.7 by Robert Chapin (miqrogroove)
 *   http://www.miqrogroove.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/

// NOT CONFIGURABLE - You will receive no support if this file is modified.
define('X_SCRIPT', 'sitemap.xml.php');
define('MAX_URL_COUNT', 50000);

require 'header.php';

$index = postedVar('index', '', FALSE, FALSE, FALSE, 'g');

switch($index) {
case '':
    makeIndex();
    break;
case 'root':
    makeRootMap();
    break;
default:
    $page = intval(substr($index, 7));
    if (substr($index, 0, 7) == 'threads') {
        makeThreadMap($page);
    } elseif (substr($index, 0, 7) == 'members') {
        makeMemberMap($page);
    } else {
        loadtemplates('');
        header('HTTP/1.0 404 Not Found');
        error('404 Not Found');
    }
    break;
}



function makeIndex() {
    //Generate sitemap index.
    global $db, $full_url, $SETTINGS;

    $fids = implode(', ', loadFIDs());
    if ($fids != '') {
        $threadwhere = "t.fid IN($fids) AND closed NOT LIKE 'moved|%'";
    } else {
        $threadwhere = "1=2";
    }
    $query = $db->query("SELECT COUNT(uid) FROM ".X_PREFIX."members WHERE lastvisit!=0 UNION ALL SELECT SUM(replies + 1) FROM ".X_PREFIX."threads AS t WHERE $threadwhere");
    $membercount = $db->result($query, 0);
    $postcount = $db->result($query, 1);
    header('Content-Type: text/xml; charset=UTF-8');
    echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n";
?>
<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
   <sitemap>
      <loc><?php echo $full_url.X_SCRIPT.'?index=root'; ?></loc>
   </sitemap>
<?php if ($postcount > 0) { ?>
   <sitemap>
      <loc><?php echo $full_url.X_SCRIPT.'?index=threads'; ?></loc>
   </sitemap>
<?php }
    //Generate additional maps if there is a huge number of posts on this board.
    $counter = 1;
    while($postcount > MAX_URL_COUNT) {
?>
   <sitemap>
      <loc><?php echo $full_url.X_SCRIPT.'?index=threads'.++$counter; ?></loc>
   </sitemap>
<?php
        $postcount -= MAX_URL_COUNT;
    }
    
    if ($membercount > 0 And $SETTINGS['memliststatus'] == 'on') {
?>
   <sitemap>
      <loc><?php echo $full_url.X_SCRIPT.'?index=members'; ?></loc>
   </sitemap>
<?php
        //Generate additional maps if there is a huge number of members on this board.
        $counter = 1;
        while($membercount > MAX_URL_COUNT) {
?>
   <sitemap>
      <loc><?php echo $full_url.X_SCRIPT.'?index=members'.++$counter; ?></loc>
   </sitemap>
<?php
            $membercount -= MAX_URL_COUNT;
        }
    }
?>
</sitemapindex>
<?php
}


function makeRootMap() {
    //Generate map of root paths and forums.
    global $db, $full_url, $SETTINGS;
    header('Content-Type: text/xml; charset=UTF-8');
    echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n";
?>
<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
   <!-- Root Paths -->
   <url>
      <loc><?php echo $full_url; ?></loc>
      <changefreq>hourly</changefreq>
   </url>
   <url>
      <loc><?php echo $full_url.'misc.php?action=login'; ?></loc>
      <changefreq>never</changefreq>
   </url>
<?php if ($SETTINGS['coppa'] == 'on') { ?>
   <url>
      <loc><?php echo $full_url.'member.php?action=coppa'; ?></loc>
      <changefreq>never</changefreq>
   </url>
<?php } if ($SETTINGS['regstatus'] == 'on') { ?>
   <url>
      <loc><?php echo $full_url.'member.php?action=reg'; ?></loc>
      <changefreq>never</changefreq>
   </url>
<?php } if ($SETTINGS['whosonlinestatus'] == 'on') { ?>
   <url>
      <loc><?php echo $full_url.'misc.php?action=online'; ?></loc>
      <changefreq>always</changefreq>
   </url>
<?php } if ($SETTINGS['onlinetoday_status'] == 'on') { ?>
   <url>
      <loc><?php echo $full_url.'misc.php?action=onlinetoday'; ?></loc>
      <changefreq>hourly</changefreq>
   </url>
<?php } if ($SETTINGS['searchstatus'] == 'on') { ?>
   <url>
      <loc><?php echo $full_url.'search.php'; ?></loc>
      <changefreq>never</changefreq>
   </url>
<?php } if ($SETTINGS['faqstatus'] == 'on') { ?>
   <url>
      <loc><?php echo $full_url.'faq.php'; ?></loc>
      <changefreq>never</changefreq>
   </url>
   <url>
      <loc><?php echo $full_url.'faq.php?page=usermaint'; ?></loc>
      <changefreq>never</changefreq>
   </url>
   <url>
      <loc><?php echo $full_url.'faq.php?page=using'; ?></loc>
      <changefreq>never</changefreq>
   </url>
   <url>
      <loc><?php echo $full_url.'faq.php?page=messages'; ?></loc>
      <changefreq>never</changefreq>
   </url>
<?php } if ($SETTINGS['todaysposts'] == 'on') { ?>
   <url>
      <loc><?php echo $full_url.'today.php'; ?></loc>
      <changefreq>daily</changefreq>
   </url>
<?php } if ($SETTINGS['stats'] == 'on') { ?>
   <url>
      <loc><?php echo $full_url.'stats.php'; ?></loc>
      <changefreq>daily</changefreq>
   </url>
<?php } if ($SETTINGS['memliststatus'] == 'on') { ?>
   <url>
      <loc><?php echo $full_url.'misc.php?action=list'; ?></loc>
      <changefreq>monthly</changefreq>
   </url>
<?php }

    echo "   <!-- Forum Groups -->\r\n";
    foreach(loadFIDs(TRUE) as $fid) {
?>
   <url>
      <loc><?php echo $full_url.'index.php?gid='.$fid; ?></loc>
      <changefreq>hourly</changefreq>
   </url>
<?php
    }

    echo "   <!-- Forums -->\r\n";
    foreach(loadFIDs() as $fid) {
?>
   <url>
      <loc><?php echo $full_url.'forumdisplay.php?fid='.$fid; ?></loc>
      <changefreq>hourly</changefreq>
   </url>
<?php
    }
?>
</urlset>
<?php
}


function makeThreadMap($page) {
    //Generate map of threads
    global $db, $full_url, $postperpage;

    $fids = implode(', ', loadFIDs());

    if ($fids != '') {
        $threadwhere = "t.fid IN($fids) AND closed NOT LIKE 'moved|%'";
    } else {
        $threadwhere = "1=2";
    }

    if ($page > 1) {
        $query = $db->query("SELECT SUM(replies + 1) FROM ".X_PREFIX."threads AS t WHERE $threadwhere");
        $postcount = $db->result($query, 0);
        if ($page <= ceil($postcount / MAX_URL_COUNT)) {
            $start = ($page - 1) * MAX_URL_COUNT;
            $limit = "LIMIT $start,".MAX_URL_COUNT;
        } else {
            loadtemplates('');
            header('HTTP/1.0 404 Not Found');
            error('404 Not Found');
        }
        $db->free_result($query);
    } else {
        $limit = 'LIMIT '.MAX_URL_COUNT;
    }

    // This nested query ensures we get all posts for the selected threads,
    // even with the limit on posts.  Hence, no need for a running sum of replies.
    $sql =
        "SELECT p.pid, p.tid, p.dateline, q3.closed
         FROM ".X_PREFIX."posts AS p
         INNER JOIN
         (
           SELECT tid, closed
           FROM
           (
             SELECT p2.tid, t.closed
             FROM ".X_PREFIX."posts AS p2
             INNER JOIN ".X_PREFIX."threads AS t USING (tid)
             WHERE $threadwhere
             ORDER BY p2.tid, p2.dateline, p2.pid
             $limit
           ) AS q2
           GROUP BY tid, closed
         ) AS q3 ON p.tid = q3.tid
         ORDER BY p.tid, p.dateline, p.pid";

    $query = $db->query($sql);

    // Prime the loop.
    $post = $db->fetch_array($query);
    $tid = '';

    header('Content-Type: text/xml; charset=UTF-8');
    echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n";
?>
<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
<?php
    while(is_array($post)) {

        // Prime the inner loop
        $postcount = 0;
        if ($post['tid'] == $tid) {
            $pageurl = '&amp;page='.++$page;
        } else {
            $page = 1;
            $pageurl = '';
            $tid = $post['tid'];
        }

        while ($postcount < $postperpage and $post['tid'] == $tid) {
            $closed = $post['closed'];
            $dateline = $post['dateline'];

            $post = $db->fetch_array($query);
            if (FALSE === $post) break;
            $postcount++;
        }

        $lastpost = gmdate('Y-m-d\\TH:i\\Z' , $dateline);
?>
   <url>
      <loc><?php echo $full_url.'viewthread.php?tid='.$tid.$pageurl; ?></loc>
      <lastmod><?php echo $lastpost; ?></lastmod>
<?php if ('yes' == $closed) { ?>
      <changefreq>never</changefreq>
<?php } ?>
   </url>
<?php
    }
?>
</urlset>
<?php
    $db->free_result($query);
}


function makeMemberMap($page) {
    //Generate map of members
    global $db, $full_url, $SETTINGS;

    if ($SETTINGS['memliststatus'] != 'on') {
        loadtemplates('');
        header('HTTP/1.0 404 Not Found');
        error('404 Not Found');
    }

    if ($page > 1) {
        $query = $db->query("SELECT COUNT(tid) FROM ".X_PREFIX."members WHERE lastvisit!=0");
        $membercount = $db->result($query, 0);
        if ($page <= ceil($membercount / MAX_URL_COUNT)) {
            $start = ($page - 1) * MAX_URL_COUNT;
            $limit = "LIMIT $start,".MAX_URL_COUNT;
        } else {
            loadtemplates('');
            header('HTTP/1.0 404 Not Found');
            error('404 Not Found');
        }
        $db->free_result($query);
    } else {
        $limit = 'LIMIT '.MAX_URL_COUNT;
    }

    $query = $db->query("SELECT username FROM ".X_PREFIX."members WHERE lastvisit!=0 ORDER BY username $limit");
    header('Content-Type: text/xml; charset=UTF-8');
    echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n";
?>
<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
<?php
    while($member = $db->fetch_array($query)) {
?>
   <url>
      <loc><?php echo $full_url.'member.php?action=viewpro&amp;member='.recodeOut($member['username']); ?></loc>
      <changefreq>monthly</changefreq>
   </url>
<?php
    }
?>
</urlset>
<?php
    $db->free_result($query);
}

//loadFIDs returns an array of forum and subforum IDs accessible by Guests.
//$groupsonly is a bool, if TRUE then an array of forum group IDs is returned instead.
function loadFIDs($groupsonly=FALSE) {
    $forums = permittedForums(forumCache(), 'thread', 'array', TRUE, 'Guest');
    $fids = array();

    if ($groupsonly) {
        $fups = array();
        foreach($forums as $forum) {
            if ($forum['type'] == 'group') {
                $fids[] = $forum['fid'];
            } elseif ($forum['type'] == 'forum') {
                $fups[] = $forum['fup'];
            } // else - subforums irrelevant
        }
        $fups = array_unique($fups);
        $fids = array_intersect($fids, $fups);
    } else {
        foreach($forums as $forum) {
            if ($forum['type'] != 'group') {
                $fids[] = $forum['fid'];
            }
        }
    }
        
    return $fids;
}
?>
